library(RGtk)
library(RGtkScrollpane)
library(gtkDevice)


panner2D =
function(range = matrix(c(1, 1, 100, 100), 2, 2), labelNames = c("df1", "df2"))
{

 h = gtkAdjustment(50, range[1,1], range[1,2], 1, 10, 10)
 v = gtkAdjustment(50, range[2,1], range[2,2], 1, 10, 10)
 sc = gtkScrollpaneNew(h, v, aspectRatio = 1)


 box = gtkVBox()

 box$PackStart(sc)

 labels = list()
 for(i in labelNames) {
   tmp = gtkHBox()
   tmp$PackStart(gtkLabel(i))
   labels[[i]] = gtkLabel("0")
   tmp$PackEnd(labels[[i]])
   box$PackStart(tmp)
 }

 showDF = function(w) {
   labels[[1]]$SetText(round(h$GetValue()))
   labels[[2]]$SetText(round(v$GetValue())) 
 }

 h$AddCallback("value-changed", showDF)
 v$AddCallback("value-changed", showDF)

 
 return(list(box = box, labels = labels, adj = list(hor = h, vert = v)))
}


DistnViewer =
function(draw, ...)
{
  b = gtkHBox()
  pan = panner2D(...)

  b$PackStart(pan$box)

  devBox = gtkHBox()

  dev = integer()
  draws =list()

  for(i in 1:2) {
    draws[[i]] = gtkDrawingArea()
    devBox$PackStart(draws[[i]])
    asGtkDevice(draws[[i]])
    dev[i] = dev.cur()
  }
 devBox$SetUsize(400, 300)
  b$PackEnd(devBox)

 h = pan$adj$hor
 v = pan$adj$vert

 currentValues = integer(2)
 cb = function(w) {
   df1 = round(h$GetValue())
   df2 = round(v$GetValue())
   if(!all(currentValues == c(df1, df2))) {
      draw(df1, df2)
      currentValues <<- c(df1, df2)
   }
 }

 environment(draw) = environment(cb)  
  
 h$AddCallback("value-changed", cb)
 v$AddCallback("value-changed", cb)

 return(list(box = b, devices = dev, drawingAreas = draws, panner = pan,
              draw = draw))
}

DistnExplorer =
function(...)
{
  draw = function(df1, df2) {
   dev.set(dev[1])
   curve(df(x, df1, df2), 0, 10, ylab="", main=paste("Density of F(", df1, ", ", df2, ")", sep=""))
   dev.set(dev[2])
   curve(pf(x, df1, df2), 0, 10, ylab="", main = paste("CDF of F(", df1, ", ", df2, ")", sep=""))   
  }

  DistnViewer(draw, ...)
}
  

QQExplorer =
function(...)
{
 
  draw = function(df1, df2) {
   vals = rf(num, df1, df2)

   dev.set(dev[1])   
   hist(vals, prob=TRUE, ylab="", xlab = "X", main=paste("Density of F(", df1, ", ", df2, ")", sep=""))
   curve(df(x, df1, df2), 0, 10, add = TRUE, col = "red")

   dev.set(dev[2])
   qqplot(vals, qf((1:num)/(num+.005), df1, df2), xlab = "X", ylab="",
            main = paste("Q-Q plot of sample of F(", df1, ", ", df2, ")", sep=""))   
 }

  tt = DistnViewer(draw)

  assign("num", 1000, envir = environment(tt$draw))

  adj = gtkAdjustment(1000, 1, 10000, 1, 100, 100)
  slider = gtkHScale(adj)
  tt$pan$box$PackEnd(slider)

  cb = function(w) {
    num <<-  as.integer(w$GetValue())
    draw(currentValues[1], currentValues[2])
  }
  environment(cb) = environment(tt$draw)
  
  adj$AddCallback("value-changed", cb)

  invisible(tt)
}  
  

if(FALSE) {
 w = gtkWindow() 
 w$Add(tt$box)
}

QQExplorer()$box




